# Copyright 2012 Leonidas Poulopoulos
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import socket
import re
from exceptions import RuntimeError
import sys

class eos(object):
    '''
    Parent Class responsible for talking the common parts of Alcatel PMD,NED(ExtOS,GENOS etc) Protocol
    '''
    MAX_MESSAGE_SIZE = 5000000
    '''
    Maximum socket message size
    '''
    _socket = None
    '''
    The TCP/IP socket over which communication takes place
    '''
    _server = None
    '''
    The IPv4 address of the NMS server
    ''' 
    _port = None
    '''
    The port we will talk to. Should be configured on the NMS Server as well
    ''' 
    _password = None
    '''
    The password. Completely and utterly unsafe but for now its should be ok
    ''' 
     
    def __init__(self, server, port, password):
        '''
        Constructor for the class.
        
        @type server: String
        @param server: The Hostname or IPv4 address of the Alcatel NMS.
        
        @type password: String
        @param password: The password for this connection.
        
        @type port: Integer
        @param port: The TCP port on the Alcatel NMS. Depending on the interface this could be 20006 (NE) or 20007 (PM)
        
        @rtype: Connection handler
        @return: A connection handler to the NMS or False in case of failure.
        '''
        if ( password != None ):
            self._password = password
        if ( re.match(r"^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}", server) ):
            self._server = server
        else:
            address = socket.gethostbyname(server)
            if ( address == server ):
                print "Error: gethostbyname() failed: No such hostname\n";
                return None
        if ( port > 0 and port < 65535 ):
            self._port = port;
        else:
            print "Error: given Port: %s is out of range\n" % port
            return None
        conn = self.connect()
        if not conn:
            print "Could not initialize connection"
            return None
        
    def __exit__(self, *args):
        '''
        Exit method to be used with 'with' statement
        
        '''
        self.disconnect()
        return False
    
    def __enter__(self):
        '''
        Enter method to be used with 'with' statement
        
        '''
        return self
    
    def send(self, message):
        '''
        Function used to send a message to Alcatel GENOS.
        Not to be called directly.
        
        @type message: Integer
        @param message: A string containing the message to be sent.
        
        @rtype: Boolean
        @return: True if sending was successfull, False otherwise.
        '''
        if ( message == None ):
            return False
        try:
            result=self._socket.sendall(message)
        except socket.error, msg:
            print "Error: socket send message %s failed: %s" %(message, msg)
            return False
        return True

    def get_part(self):
        '''
        Function used to send a message to Alcatel GENOS.
        Not to be called directly.
        
        @rtype: Boolean 
        @return: True if sending was successfull, False otherwise.
        '''
        try:
            out = self._socket.recv(self.MAX_MESSAGE_SIZE)
        except socket.error, msg:
            print "Error: socket receive message failed: %s" %msg
            return False
        return out

    def get(self):
        '''
        Function use to get a message for the NMS.
        Not to be called directly.
        
        @rtype:  String
        @return: A string representing the message sent by the NMS.
        '''
        msg = ''
        sys.stdout.write("Getting reply.")
        sys.stdout.flush()
        while True:
            chunk = self.get_part()
            sys.stdout.write(".")
            sys.stdout.flush()
            if not chunk:
                raise Exception("socket connection broken")
            msg = msg + chunk
            if self._is_data_end(chunk):
                break
        print "done!"
        return msg

    def _is_data_end(self, string):
        '''
        Function finding if the end of data string has been sent from the NMS.
        Not to be called directly.
        
        @type string: String
        @param string: The string to be checked for the end of data string.
        
        @rtype:  Boolean
        @return: True if end of data string has been found, False otherwise.        
        '''
        if ( re.match(r"DATA_END_NOTIF\[\]",string) ):
            return True
        return False

    def _open_connection(self):
        '''
        Function opening a socket and connecting to the NMS. 
        Not to be called directly.
        
        @rtype: Boolean
        @return: True if successful, False otherwise.
        '''
        try:
            s = socket.socket(socket.AF_INET,socket.SOCK_STREAM,socket.SOL_TCP)
        except socket.error, msg:
            print "Error: socket creation failed: %s" %msg
            return False
        self._socket = s
        try:
            result = self._socket.connect((self._server, int(self._port)))
        except socket.error, msg:
            print "Error: socket connection failed: %s" %msg
            return False
        return True

    def _close_connection(self):
        '''
        Function closing the socket connection.
        Not to be called directly.
        
        @rtype: None
        @return: Nothing.
        '''
        self._socket.close()

    
    def _authenticate(self):
        '''
        Function authenticating to the NMS.
        Not to be called directly.
        
        @rtype:  Boolean
        @return: True if authentication is successful , False otherwise.
        '''
        auth = "CON_REQ[KEY=%s]" %(self._password)
        if ( self.send(auth) == False ):
            return False
        result = self.get_part()
        if ( result == 'CON_CONF[]' ):
            return True
        elif ( result == "CON_REJ[]" ):
            print "Error: Authentication failed\n"
            return False
        else:
            print "Error: unknown error in socket communication"
            return False

    def connect(self):
        '''
        Function connecting and authenticating to the NMS.
    
        @rtype:  Boolean
        @return: True if authentication is successful , False otherwise.
        '''
        if ( self._open_connection() == False ):
            return False
        if ( self._authenticate() == False ):
            return False
        return True


    def disconnect(self):
        '''
        Function sending disconnect message and closing the connection.
        
        @rtype:  Boolean
        @return: True if correctly disconnected, False otherwise.
        '''
        conn = "DISCON_NOTIF[]"
        if ( self.send(conn) == False ):
            return False
        self._close_connection()
        return True

